local ADDON_NAME, Core = ...

Core.defaults = {
	["BGOpacity"] = 30,
	["HistoryExpiry"] = 10,
	["HubLocked"] = false,
	["HubToastAnchor"] = "TOPRIGHT",
	["CloseOnESC"] = true,
	["PartyEnabled"] = false,
	["GuildEnabled"] = false,
	["NearbyChatEnabled"] = true,
}

local contextMenu
local UI = Core.UI


local function setExpiry (value)
	SESMSG_Profile["HistoryExpiry"] = value
end


local function checkExpiry (value)
	return SESMSG_Profile["HistoryExpiry"] == value
end


local function setOpacity (value)
	UI.BG:SetColorTexture(0, 0, 0, value)
	SESMSG_Profile["BGOpacity"] = value
end


local function checkOpacity (value)
	return SESMSG_Profile["BGOpacity"] == value
end


local function toggle_CloseOnESCPressed ()
	if not SESMSG_Profile["CloseOnESC"] then
		table.insert(UISpecialFrames, UI:GetName())

	else
		for i, frameName in ipairs(UISpecialFrames) do
			if frameName == UI:GetName() then
				table.remove(UISpecialFrames, i)
			end
		end
	end
	
	SESMSG_Profile["CloseOnESC"] = not SESMSG_Profile["CloseOnESC"]	
end


local function toggle_PartyChat ()
	SESMSG_Profile["PartyEnabled"] = not SESMSG_Profile["PartyEnabled"]
	
	if SESMSG_Profile["PartyEnabled"] then
		Core.UI:RegisterEvent("CHAT_MSG_PARTY")
		Core.UI:RegisterEvent("CHAT_MSG_PARTY_LEADER")
		Core.UI:RegisterEvent("GROUP_JOINED")
		Core.LoadChannels()
	else
		Core.UI:UnregisterEvent("CHAT_MSG_PARTY")
		Core.UI:UnregisterEvent("CHAT_MSG_PARTY_LEADER")
		Core.UI:UnregisterEvent("GROUP_JOINED")
		
		if Core.App:Get("PARTY") then
			Core.App.active["PARTY"]:Deactivate()
		end
	end
end


local function toggle_GuildChat ()
	SESMSG_Profile["GuildEnabled"] = not SESMSG_Profile["GuildEnabled"]
	
	if SESMSG_Profile["GuildEnabled"] then
		Core.UI:RegisterEvent("CHAT_MSG_GUILD")
		Core.App:AddChannel("Guild")
	else
		Core.UI:UnregisterEvent("CHAT_MSG_GUILD")
		
		if Core.App:Get("GUILD") then
			Core.App.active["GUILD"]:Deactivate()
		end
	end
end


local function toggle_NearbyChat ()
	SESMSG_Profile["NearbyChatEnabled"] = not SESMSG_Profile["NearbyChatEnabled"]
	
	if SESMSG_Profile["NearbyChatEnabled"] then
		Core.UI:RegisterEvent("CHAT_MSG_SAY")
		Core.UI:RegisterEvent("CHAT_MSG_TEXT_EMOTE")
		Core.UI:RegisterEvent("CHAT_MSG_EMOTE")
		Core.UI:RegisterEvent("ZONE_CHANGED")
	
	else
		Core.UI:UnregisterEvent("CHAT_MSG_SAY")
		Core.UI:UnregisterEvent("CHAT_MSG_TEXT_EMOTE")
		Core.UI:UnregisterEvent("CHAT_MSG_EMOTE")
		Core.UI:UnregisterEvent("ZONE_CHANGED")
	end
end


local function createContextMenu ()
	titleGradient = "|cFF417dc1Sess|r|cFF2effebions|r |cFFf28500Mess|r|cFFeb2effenger|r"
	contextMenu = {
		{text=titleGradient.." v".. Core.VERSION, isTitle=true, notCheckable=true},
		{
			text="Background Opacity",
			hasArrow=true,
			notCheckable=true,
			keepShownOnClick=true,
			menuList={
				{text="30 %", value=.30, checked=function(self) return checkOpacity(self.value) end, func=function(self) setOpacity(self.value) end},
				{text="70 %", value=.70, checked=function(self) return checkOpacity(self.value) end, func=function(self) setOpacity(self.value) end},
				{text="100 %", value=1, checked=function(self) return checkOpacity(self.value) end, func=function(self) setOpacity(self.value) end},
			},
		},
		{
			text="Close UI On ESC",
			checked=function() return SESMSG_Profile["CloseOnESC"] end,
			func=toggle_CloseOnESCPressed,
			keepShownOnClick=true,
		},
		{
			text="Toggle Resize Button",
			func=function() UI.resizeBtn:SetShown(not UI.resizeBtn:IsShown()) end,
			notCheckable=true,
			keepShownOnClick=true,
			tooltipTitle="Tip!",
			tooltipText="Hold down CTRL to show resize button",
			tooltipOnButton=true,
		},
		{
			text="Change Chat Colours",
			notCheckable=true,
			func=function() ChatConfigFrame:Show() end,			
		},
		{text="|TInterface/COMMON/UI-TooltipDivider:8:144|t", notClickable=true, notCheckable=true},
		{text="Channels", isTitle=true, notCheckable=true},
		{
			text="Enable Party Chat",
			keepShownOnClick=true,
			checked=function() return SESMSG_Profile["PartyEnabled"] end,
			func=toggle_PartyChat
		},
		{
			text="Enable Guild Chat",
			keepShownOnClick=true,
			checked=function() return SESMSG_Profile["GuildEnabled"] end,
			func=toggle_GuildChat
		},
		{
			text="Enable Nearby Chat",
			keepShownOnClick=true,
			tooltipTitle="Nearby Chat",
			tooltipText="Say & Emote",
			tooltipOnButton=true,
			checked=function() return SESMSG_Profile["NearbyChatEnabled"] end,
			func=toggle_NearbyChat,
			
		},
		{text="|TInterface/COMMON/UI-TooltipDivider:8:144|t", notClickable=true, notCheckable=true},
		{text="History", isTitle=true, notCheckable=true},
		{
			text="Open History Viewer",
			notCheckable=true,
			func=function() Core.History:Initialize(); Core.History:Show() end
		},
		{
			text= "History Expiry",
			hasArrow=true,
			notCheckable=true,
			keepShownOnClick=true,
			menuList={
				{text="10 Days", value=10, checked=function(self) return checkExpiry(self.value) end, func=function(self) setExpiry(self.value) end},
				{text="15 Days", value=15, checked=function(self) return checkExpiry(self.value) end, func=function(self) setExpiry(self.value) end},
				{text="20 Days", value=20, checked=function(self) return checkExpiry(self.value) end, func=function(self) setExpiry(self.value) end},
				{text="30 Days", value=30, checked=function(self) return checkExpiry(self.value) end, func=function(self) setExpiry(self.value) end},
			}
		},
		{text="Cancel", notCheckable=true},
	}
	
end


function Core.LoadPreferences ()
	UI.BG:SetColorTexture(0, 0, 0, SESMSG_Profile["BGOpacity"])

	if SESMSG_Profile["HubLocked"] then
		Core.hub:RegisterForDrag()
	else
		Core.hub:RegisterForDrag("LeftButton")
		Core.hub:Preview("ON")
	end
	
	
	if SESMSG_Profile["PartyEnabled"] then
		Core.UI:RegisterEvent("CHAT_MSG_PARTY")
		Core.UI:RegisterEvent("CHAT_MSG_PARTY_LEADER")
		Core.UI:RegisterEvent("GROUP_JOINED")
	end
	
	
	if SESMSG_Profile["GuildEnabled"] then
		Core.UI:RegisterEvent("CHAT_MSG_GUILD")
	end
	
	
	if SESMSG_Profile["NearbyChatEnabled"] then
		Core.UI:RegisterEvent("CHAT_MSG_SAY")
		Core.UI:RegisterEvent("CHAT_MSG_TEXT_EMOTE")
		Core.UI:RegisterEvent("CHAT_MSG_EMOTE")
		Core.UI:RegisterEvent("ZONE_CHANGED")
	end
	
	
	if SESMSG_Profile["CloseOnESC"] then
		table.insert(UISpecialFrames, UI:GetName())

	else
		for i, frameName in ipairs(UISpecialFrames) do
			if frameName == UI:GetName() then
				table.remove(UISpecialFrames, i)
			end
		end
	end
	
	
	createContextMenu()
	
end


function Core.ShowOptions ()
	EasyMenu(contextMenu, UI.cardMenu, UI.header.optionsBtn, 10, 110, "MENU")
end

